@@ -6,13 +6,16 @@ package org.torproject.descriptor.impl;
 import org.torproject.descriptor.BridgeExtraInfoDescriptor;
 import org.torproject.descriptor.DescriptorParseException;
 
+import java.io.File;
+
 public class BridgeExtraInfoDescriptorImpl
     extends ExtraInfoDescriptorImpl implements BridgeExtraInfoDescriptor {
 
   protected BridgeExtraInfoDescriptorImpl(byte[] descriptorBytes,
-      int[] offsetAndLimit, boolean failUnrecognizedDescriptorLines)
-      throws DescriptorParseException {
-    super(descriptorBytes, offsetAndLimit, failUnrecognizedDescriptorLines);
+      int[] offsetAndLimit, File descriptorFile,
+      boolean failUnrecognizedDescriptorLines) throws DescriptorParseException {
+    super(descriptorBytes, offsetAndLimit, descriptorFile,
+        failUnrecognizedDescriptorLines);
   }
 }
 
