@@ -21,6 +21,15 @@ import java.util.regex.Pattern;
 
 import javax.xml.bind.DatatypeConverter;
 
+/**
+ * Parse helper for descriptor contents.
+ *
+ * <p>Naming convention: methods starting with {@code parse} return valid and
+ * sometimes changed data items (e.g. string to port int), methods starting with
+ * {@code verify} only check data items without returning them, and methods
+ * starting with {@code convert} return checked and explicitly changed data
+ * items (e.g. base64 to hex encoding).</p>
+ */
 public class ParseHelper {
 
   private static Pattern keywordPattern = Pattern.compile("^[A-Za-z0-9-]+$");
@@ -281,7 +290,12 @@ public class ParseHelper {
   private static Pattern twentyByteBase64Pattern =
       Pattern.compile("^[0-9a-zA-Z+/]{27}$");
 
-  protected static String parseTwentyByteBase64String(String line,
+  protected static void verifyTwentyByteBase64String(String line,
+      String base64String) throws DescriptorParseException {
+    convertTwentyByteBase64StringToHex(line, base64String);
+  }
+
+  protected static String convertTwentyByteBase64StringToHex(String line,
       String base64String) throws DescriptorParseException {
     if (!twentyByteBase64Pattern.matcher(base64String).matches()) {
       throw new DescriptorParseException("'" + base64String
@@ -296,7 +310,12 @@ public class ParseHelper {
   private static Pattern thirtyTwoByteBase64Pattern =
       Pattern.compile("^[0-9a-zA-Z+/]{43}$");
 
-  protected static String parseThirtyTwoByteBase64String(String line,
+  protected static void verifyThirtyTwoByteBase64String(String line,
+      String base64String) throws DescriptorParseException {
+    convertThirtyTwoByteBase64StringToHex(line, base64String);
+  }
+
+  private static String convertThirtyTwoByteBase64StringToHex(String line,
       String base64String) throws DescriptorParseException {
     if (!thirtyTwoByteBase64Pattern.matcher(base64String).matches()) {
       throw new DescriptorParseException("'" + base64String
