@@ -6,13 +6,16 @@ package org.torproject.descriptor.impl;
 import org.torproject.descriptor.DescriptorParseException;
 import org.torproject.descriptor.RelayExtraInfoDescriptor;
 
+import java.io.File;
+
 public class RelayExtraInfoDescriptorImpl
     extends ExtraInfoDescriptorImpl implements RelayExtraInfoDescriptor {
 
   protected RelayExtraInfoDescriptorImpl(byte[] descriptorBytes,
-      int[] offsetAndLimit, boolean failUnrecognizedDescriptorLines)
-      throws DescriptorParseException {
-    super(descriptorBytes, offsetAndLimit, failUnrecognizedDescriptorLines);
+      int[] offsetAndLimit, File descriptorFile,
+      boolean failUnrecognizedDescriptorLines) throws DescriptorParseException {
+    super(descriptorBytes, offsetAndLimit, descriptorFile,
+        failUnrecognizedDescriptorLines);
     this.calculateDigestSha1Hex(Key.EXTRA_INFO.keyword + SP,
         NL + Key.ROUTER_SIGNATURE.keyword + NL);
     this.calculateDigestSha256Base64(Key.EXTRA_INFO.keyword + SP,
