@@ -1,5 +1,6 @@
-/* Copyright 2015--2016 The Tor Project
+/* Copyright 2015--2017 The Tor Project
  * See LICENSE for licensing information */
+
 package org.torproject.descriptor.impl;
 
 import static org.junit.Assert.assertEquals;
@@ -8,16 +9,16 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 
-import java.util.SortedMap;
-
 import org.junit.Test;
 
+import java.util.SortedMap;
+
 public class DescriptorCollectorImplTest {
 
   private static final String REMOTE_DIRECTORY_CONSENSUSES =
       "/recent/relay-descriptors/consensuses/";
 
-  @Test()
+  @Test
   public void testOneFile() {
     String remoteFilename = "2015-05-24-12-00-00-consensus";
     String directoryListing = "<tr><td valign=\"top\">"
@@ -37,7 +38,7 @@ public class DescriptorCollectorImplTest {
         remoteFiles.get(remoteFiles.firstKey()));
   }
 
-  @Test()
+  @Test
   public void testSameFileTwoTimestampsLastWins() {
     String remoteFilename = "2015-05-24-12-00-00-consensus";
     String firstTimestamp = "2015-05-24 12:04";
@@ -60,7 +61,7 @@ public class DescriptorCollectorImplTest {
         remoteFiles.get(remoteFiles.firstKey()));
   }
 
-  @Test()
+  @Test
   public void testSubDirectoryOnly() {
     String directoryListing = "<tr><td valign=\"top\">"
         + "<img src=\"/icons/folder.gif\" alt=\"[DIR]\"></td><td>"
@@ -74,7 +75,7 @@ public class DescriptorCollectorImplTest {
     assertTrue(remoteFiles.isEmpty());
   }
 
-  @Test()
+  @Test
   public void testParentDirectoryOnly() {
     String directoryListing = "<tr><td valign=\"top\">"
         + "<img src=\"/icons/back.gif\" alt=\"[DIR]\"></td><td>"
@@ -88,7 +89,7 @@ public class DescriptorCollectorImplTest {
     assertTrue(remoteFiles.isEmpty());
   }
 
-  @Test()
+  @Test
   public void testUnexpectedDateFormat() {
     String directoryListing = "<tr><td valign=\"top\">"
         + "<img src=\"/icons/unknown.gif\" alt=\"[   ]\"></td><td>"
@@ -103,7 +104,7 @@ public class DescriptorCollectorImplTest {
     assertTrue(remoteFiles.isEmpty());
   }
 
-  @Test()
+  @Test
   public void testInvalidDate() {
     String directoryListing = "<tr><td valign=\"top\">"
         + "<img src=\"/icons/unknown.gif\" alt=\"[   ]\"></td><td>"
