@@ -1,23 +1,30 @@
-/* Copyright 2015--2016 The Tor Project
+/* Copyright 2015--2017 The Tor Project
  * See LICENSE for licensing information */
+
 package org.torproject.descriptor.impl;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
-import java.util.HashMap;
-import java.util.Map;
-
-import org.junit.Test;
 import org.torproject.descriptor.DescriptorParseException;
 import org.torproject.descriptor.ExitListEntry;
 
+import org.junit.Rule;
+import org.junit.Test;
+import org.junit.rules.ExpectedException;
+
+import java.util.HashMap;
+import java.util.Map;
+
 public class ExitListImplTest {
 
-  @Test()
+  @Rule
+  public ExpectedException thrown = ExpectedException.none();
+
+  @Test
   public void testAnnotatedInput() throws Exception {
     ExitListImpl result = new ExitListImpl((tordnselAnnotation + input)
-        .getBytes("US-ASCII"), fileName, false);
+        .getBytes("US-ASCII"), null, fileName, false);
     assertEquals("Expected one annotation.", 1,
         result.getAnnotations().size());
     assertEquals(tordnselAnnotation.substring(0, 18),
@@ -31,11 +38,11 @@ public class ExitListImplTest {
         result.getEntries().size());
   }
 
-  @Test()
+  @Test
   public void testMultipleOldExitAddresses() throws Exception {
     ExitListImpl result = new ExitListImpl(
         (tordnselAnnotation + multiExitAddressInput)
-        .getBytes("US-ASCII"), fileName, false);
+        .getBytes("US-ASCII"), null, fileName, false);
     assertTrue("Unrecognized lines: " + result.getUnrecognizedLines(),
         result.getUnrecognizedLines().isEmpty());
     assertEquals("Found: " + result.getExitListEntries(),
@@ -57,11 +64,11 @@ public class ExitListImplTest {
     assertTrue("Map: " + testMap, testMap.isEmpty());
   }
 
-  @Test()
+  @Test
   public void testMultipleExitAddresses() throws Exception {
     ExitListImpl result = new ExitListImpl(
         (tordnselAnnotation + multiExitAddressInput)
-        .getBytes("US-ASCII"), fileName, false);
+        .getBytes("US-ASCII"), null, fileName, false);
     assertTrue("Unrecognized lines: " + result.getUnrecognizedLines(),
         result.getUnrecognizedLines().isEmpty());
     Map<String, Long> map = result.getEntries()
@@ -72,16 +79,20 @@ public class ExitListImplTest {
     assertTrue("Map: " + map, map.containsKey("81.7.17.173"));
   }
 
-  @Test(expected = DescriptorParseException.class)
+  @Test
   public void testInsufficientInput0() throws Exception {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage("Missing 'ExitAddress' line in exit list entry.");
     new ExitListImpl((tordnselAnnotation + insufficientInput[0])
-        .getBytes("US-ASCII"), fileName, false);
+        .getBytes("US-ASCII"), null, fileName, false);
   }
 
-  @Test(expected = DescriptorParseException.class)
+  @Test
   public void testInsufficientInput1() throws Exception {
+    this.thrown.expect(DescriptorParseException.class);
+    this.thrown.expectMessage("Missing 'Published' line in exit list entry.");
     new ExitListImpl((tordnselAnnotation + insufficientInput[1])
-        .getBytes("US-ASCII"), fileName, false);
+        .getBytes("US-ASCII"), null, fileName, false);
   }
 
   private static final String tordnselAnnotation = "@type tordnsel 1.0\n";
